﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.BusinessLogicContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AutomobilePlant
{
    public partial class FormShopSell : Form
    {
        private readonly ILogger _logger;
        private readonly ICarLogic _logicCar;
        private readonly IShopLogic _logicShop;

        public FormShopSell(ILogger<FormShopSell> logger, ICarLogic logicCar, IShopLogic logicShop)
        {
            InitializeComponent();
            _logger = logger;
            _logicCar = logicCar;
            _logicShop = logicShop;
        }

        private void FormShopSell_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка авто для продажи");
            try
            {
                var list = _logicCar.ReadList(null);
                if (list != null)
                {
                    comboBoxCar.DisplayMember = "CarName";
                    comboBoxCar.ValueMember = "Id";
                    comboBoxCar.DataSource = list;
                    comboBoxCar.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка авто");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSell_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxCar.SelectedValue == null)
            {
                MessageBox.Show("Выберите авто", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Создание продажи");
            try
            {
                var operationResult = _logicShop.SellCar(
                    new CarBindingModel
                    {
                        Id = Convert.ToInt32(comboBoxCar.SelectedValue)
                    },
                    Convert.ToInt32(textBoxCount.Text)
                );
                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании продажи. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания продажи");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
