﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.SearchModels;
using AutomobilePlantContracts.StorageContracts;
using AutomobilePlantContracts.ViewModels;
using AutomobilePlantDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AutomobilePlantDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new AutomobilePlantDatabase();
			return context.Orders.Include(x => x.Car).Include(x => x.Client).FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
		}

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue && !model.ClientId.HasValue)
            {
                return new();
            }
            using var context = new AutomobilePlantDatabase();
            if (model.ClientId.HasValue)
            {
                return context.Orders
                    .Include(x => x.Car)
					.Include(x => x.Client)
					.Where(x => x.ClientId == model.ClientId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return context.Orders
			.Include(x => x.Client)
			.Include(x => x.Car)
			.Where(x => x.Id == model.Id || model.DateFrom <= x.DateCreate && x.DateCreate <= model.DateTo)
            .Select(x => x.GetViewModel)
            .ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new AutomobilePlantDatabase();
			return context.Orders.Include(x => x.Car).Include(x => x.Client).Select(x => x.GetViewModel).ToList();
		}

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new AutomobilePlantDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
			return context.Orders.Include(x => x.Car).Include(x => x.Client).FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel;
		}

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new AutomobilePlantDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
			return context.Orders.Include(x => x.Car).Include(x => x.Client).FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
		}
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new AutomobilePlantDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
