﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.SearchModels;
using AutomobilePlantContracts.StorageContracts;
using AutomobilePlantContracts.ViewModels;
using AutomobilePlantDatabaseImplement.Models;
using AutomobilePlantDataModels.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AutomobilePlantDatabaseImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return new();
            }
            using var context = new AutomobilePlantDatabase();
            return context.Shops.Include(x => x.Cars).ThenInclude(x => x.Car).FirstOrDefault(x =>
                (!string.IsNullOrEmpty(model.Name) && x.Name == model.Name) ||
                (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            using var context = new AutomobilePlantDatabase();
            return context.Shops.Include(x => x.Cars).ThenInclude(x => x.Car).Where(x => x.Name.Contains(model.Name)).ToList().Select(x => x.GetViewModel).ToList();
        }

        public List<ShopViewModel> GetFullList()
        {
            using var context = new AutomobilePlantDatabase();
            return context.Shops.Include(x => x.Cars).ThenInclude(x => x.Car).ToList().Select(x => x.GetViewModel).ToList();
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            using var context = new AutomobilePlantDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var newShop = Shop.Create(context, model);
                if (newShop == null)
                {
                    return null;
                }
                if (context.Shops.Any(x => x.Name == newShop.Name))
                {
                    throw new Exception("Название магазина уже занято");
                }

                context.Shops.Add(newShop);
                context.SaveChanges();
                transaction.Commit();
                return newShop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            using var context = new AutomobilePlantDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var shop = context.Shops.Include(x => x.Cars).FirstOrDefault(x => x.Id == model.Id);
                if (shop == null)
                {
                    return null;
                }
                shop.Update(model);
                context.SaveChanges();
                if (model.ShopCars.Count > 0)
                {
                    shop.UpdateCars(context, model);
                }
                transaction.Commit();
                return shop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            using var context = new AutomobilePlantDatabase();
            var shop = context.Shops.Include(x => x.Cars).FirstOrDefault(x => x.Id == model.Id);
            if (shop != null)
            {
                context.Shops.Remove(shop);
                context.SaveChanges();
                return shop.GetViewModel;
            }
            return null;
        }

        public bool SellCar(ICarModel model, int count)
        {
            using var context = new AutomobilePlantDatabase();
            using var transaction = context.Database.BeginTransaction();
            foreach (var shopCars in context.ShopCars.Where(x => x.CarId == model.Id))
            {
                var min = Math.Min(count, shopCars.Count);
                shopCars.Count -= min;
                count -= min;
                if (count <= 0)
                {
                    break;
                }
            }
            if (count == 0)
            {
                context.SaveChanges();
                transaction.Commit();
            }
            else
            {
                transaction.Rollback();
            }
            if (count > 0)
            {
                return false;
            }
            return true;
        }
    }
}
