﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.SearchModels;
using AutomobilePlantContracts.ViewModels;
using AutomobilePlantShopApp.Models;
using Microsoft.AspNetCore.Mvc;
using System.Diagnostics;
using System.Xml.Linq;

namespace AutomobilePlantShopApp.Controllers
{
	public class HomeController : Controller
	{
		private readonly ILogger<HomeController> _logger;

		public HomeController(ILogger<HomeController> logger)
		{
			_logger = logger;
		}

		public IActionResult Index()
		{
			if (!APIClient.isAuth)
			{
				return Redirect("~/Home/Enter");
			}
			return
			View(APIClient.GetRequest<List<ShopViewModel>>($"api/shop/getshoplist"));
		}

		public IActionResult Privacy()
		{
			if (!APIClient.isAuth)
			{
				return Redirect("~/Home/Enter");
			}
			return View();
		}

		[HttpGet]
		public IActionResult Enter()
		{
			return View();
		}

		[HttpPost]
		public void Enter(string password)
		{
			if (!APIClient.Login(password))
			{
				throw new Exception("Неверный пароль");
			}
			Response.Redirect("Index");
		}
		[HttpGet]
		public IActionResult Create()
		{
			return View();
		}
		[HttpPost]
		public void Create(string name, string address, DateTime openingDate, int count)
		{
			if (!APIClient.isAuth)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}
			if (count <= 0)
			{
				throw new Exception("Максимальное число изделий в магазине должно быть больше нуля");
			}
			APIClient.PostRequest("api/shop/createshop", new ShopBindingModel
			{
				Name = name,
				MaxCountCars = count,
				Address = address,
				OpeningDate = openingDate
			});
			Response.Redirect("Index");
		}

		[HttpGet]
		public IActionResult Delete()
		{
			ViewBag.Shops = APIClient.GetRequest<List<ShopViewModel>>("api/shop/getshoplist");
			return View();
		}
		[HttpPost]
		public void Delete(int shop)
		{
			if (!APIClient.isAuth)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}

			APIClient.PostRequest($"api/shop/deleteshop", new ShopBindingModel { Id = shop });
			Response.Redirect("Index");
		}
		[HttpGet]
		public IActionResult Update()
		{
			ViewBag.Shops = APIClient.GetRequest<List<ShopViewModel>>("api/shop/getshoplist");
			return View();
		}
		[HttpPost]
		public void Update(int shop, string name, string address, DateTime openingDate, int count)
		{
			if (!APIClient.isAuth)
			{
				throw new Exception("Вы как суда попали? Суда вход только авторизованным");
			}

			APIClient.PostRequest($"api/shop/updateshop", new ShopBindingModel
			{
				Id = shop,
				Name = name,
				Address = address,
				OpeningDate = openingDate,
				MaxCountCars = count
			}
			);
			Response.Redirect("Index");
		}

        [HttpGet]
        public ShopViewModel? GetShop(int shopId)
        {
            return APIClient.GetRequest<ShopViewModel>($"api/shop/getshop?shopId={shopId}");
        }

        public IActionResult Supply()
        {

            ViewBag.Shops = APIClient.GetRequest<List<ShopViewModel>>("api/shop/getshoplist");
            ViewBag.Cars = APIClient.GetRequest<List<CarViewModel>>("api/main/getcarlist");
            return View();
        }
        [HttpPost]
        public void Supply(int shopId, int carId, int count)
        {
            if (!APIClient.isAuth)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }

            APIClient.PostRequest($"api/shop/supplycarstoshop", (new ShopSearchModel { Id = shopId }, new CarBindingModel { Id = carId }, count));
            Response.Redirect("Index");
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
		public IActionResult Error()
		{
			return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
		}
	}
}